<?php
/**
* set_env.php
* @author Luc Saillard <luc@saillard.org>
* @version 0.1
*
* This script sets up smarty, pear, and all other
* global settings for a php application.
*/

// Create a wrapper class extended from Smarty
class NavigationTree
{
  var $static_navigation_tree = array (
    array ( "Home", "/index.php" ),
    array ( "News", "/news.php"  ),
  );

  function NavigationTree()
  {
    // load translations (if needed)
    $GLOBALS['_NG_LANGUAGE_']->loadCurrentTranslationTable();
  }

  function build_tree()
  {
    $this->nav_tree = $this->__build_tree(WWW_DIR,'');
  }

  # Build the navigation tree array to be display in a template
  # url is the current url we want to hilight
  function build_navigation_tree($url)
  {
    if (!isset($this->nav_tree))
      $this->build_tree($url);
    $this->nav = array();

    # Insert static navigation items
    foreach ($this->static_navigation_tree as $key => $entry) {
      $item = array();
      $item['url'] = $entry[1];
      $item['title'] = $GLOBALS['_NG_LANGUAGE_']->getTranslation($entry[0]);
      $item['level'] = 0;
      $this->nav[] = $item;
    }
    
    $this->current_page_level = 0;
    $this->current_url = $url;
    $this->explode_url = explode('/',dirname($url));
    $this->__build_nav_array($this->nav_tree,0);
    return $this->nav;
  }

  # Only build the navigation array, but recurse itself (internal function)
  function __build_nav_array($tree,$level)
  {
    # Root directory is special ...
    if ($level > 0 )
    {
      # Check if we need to recurse into the directory
      # We display only subfolders, if the current page view is in
      # this subfolders. It's not the best method to do it, but it's works
      $pos_explode = explode('/',$tree['url']);
      for ($i=0;$i<$level;$i++)
      {
	if ($this->explode_url[$i] != $pos_explode[$i])
	  break;
      }
      if ($i < $level)
	return;

      $item = array();
      $item['url'] = $tree['url'];
      $item['title'] = $tree['document_nav_title'];
      $item['level'] = $level;
      if ($tree['url'] == dirname($this->current_url).'/') {
	$item['class'] = " currentNavItem";
	$this->current_page_level = $level;
      }
      $this->nav[] = $item;
    }
    # Then recurse on each subfolders
    if (isset($tree['folders']) && count($tree['folders']>0)) {
      foreach ($tree['folders'] as $key => $subtree) {
	$this->__build_nav_array($subtree,$level+1);
      }
    }

  }

  # Recurse into each directory and build a navigation tree
  # All subfolders will be sorted into using there name or there title
  function __build_tree($real_path,$relative_path)
  {
    $tree = array();
    $folder = $real_path . $relative_path;

    if (!is_dir($folder))
      return $tree;

    $fd = opendir($folder);
    if (!$fd)
      return $tree;

    $tree = $this->get_folder_metadata($folder);
    if (count($tree)<1)
      return $tree;
    $tree['url'] = "$relative_path/";
    $tree['short_url'] = end(explode('/',$relative_path));
    $tree['folders'] = array();
    while (false !== ($entry = readdir($fd))) {
      # Skip folders that begins with a dot, and skip files too
      if ( $entry[0]=='.' || is_file("$real_path/$relative_path/$entry") )
	continue;
      $new_tree = $this->__build_tree($real_path,"$relative_path/$entry");
      if (count($new_tree)>0)
	$tree['folders'][] = $new_tree;
    }
    closedir($fd);
    if (count($tree['folders'])<1) {
      unset($tree['folders']);
    } else {
      usort($tree['folders'],array($this,"cmp_title"));
    }

    return $tree;
  }

  function get_folder_metadata($folder)
  {
    # Do not index folders with a file named '.nodisplay'
    if (file_exists("$folder/.noindex"))
      return array();

    # try to open index.php.metadata
    $metadatas = $this->read_content("$folder/index.php.metadata");

    return $metadatas;
  }

  /*
   * The format us by the file is similar to http protocol.
   * 2 sections: headers and text
   * text section is separated from the header by an empty line.
   * header is separated by newline and contains "key: value" format
   * key will be the variable name use in the template
   * value can be any value
   */
  function read_content($file)
  {
    $metadatas = array();
    $metadatas['document_nav_title'] = $GLOBALS['_NG_LANGUAGE_']->getTranslation(end(explode('/',dirname($file))));
    $metadatas["document_date"] = filemtime(dirname($file));

    if (!file_exists($file))
      return $metadatas;

    $fd = fopen($file,"r");
    if (!$fd)
      return $metadatas;

    unset($metadatas['document_nav_title']);
    while (!feof($fd)) {
      $buffer = fgets($fd,4096);
      if (ereg("^([a-zA-Z0-9_-]+): (.*)$", $buffer, $regs)) {
	$key = $regs[1];
	$metadatas[$key] = $GLOBALS['_NG_LANGUAGE_']->getTranslation(trim($regs[2]));
      } else {
	break;
      }
    }
    fclose($fd);
    $metadatas["document_date"] = filemtime($file);
    if (!isset($metadatas['document_nav_title'])) {
      $metadatas['document_nav_title'] = $metadatas['document_title'];
    }
    return $metadatas;
  }

  function cmp_title($a, $b) 
  {
    return strcasecmp($a['document_nav_title'],$b['document_nav_title']);
  }

  function build_navigation_bar($url)
  {
    $parts = array();
    $url_explode = explode('/',dirname($url));
    $folders_list = $this->nav_tree['folders'];
    for ($i=1; count($url_explode) ; $i++)
    {
      $find_new_subfolders = 0;
      # Check all folders
      foreach ($folders_list as $key => $folder) {

	if ($folder['short_url'] == $url_explode[$i]) {
	  if ($folder['document_nav_title'][0] == '/') {
	    $title = $folder['short_url'];
	  } else {
	    $title = $folder['document_nav_title'];
	  }
	    $title = $folder['short_url'];
	  $parts[] = array("url"   => $folder['url'],
	                   "title" => $title);
	  $folders_list = $folder['folders'];
	  $find_new_subfolders = 1;
	  break;
	}

      }
      if ($find_new_subfolders == 0 || !isset($folders_list)) {
	break;
      }
    }
    return $parts;
  }

   
  


  

} /* class NavigationTree */

?>
