<?php

/**
* set_env.php
* @author Luc Saillard <luc@saillard.org>
* @version 0.1
*
* This script sets up smarty, pear, and all other
* global settings for a php application.
*/


// Create a wrapper class extended from Smarty
class Page extends Smarty
{
   // $cache and $cache_lifetime are the two main variables
  // that control caching within Smarty
  function Page($template_name="plone_look", $cache = true, $cache_lifetime = 3600)
  {
    // Change the default template directories
    $this->set_template($template_name);
    // Run Smarty's constructor
    $this->Smarty();

    // Change default caching behavior
    $this->compile_check = false;
    $this->force_compile = false;
    $this->caching = $cache;
    $this->cache_lifetime = $cache_lifetime;
    $this->cache_modified_check = true;

    // Multilanguage Support
    // use $smarty->language->setLocale() to change the language of your template
    //     $smarty->loadTranslationTable() to load custom translation tables
    $this->language = new Language($locale); // create a new language object
    $GLOBALS['_NG_LANGUAGE_'] =& $this->language;
    $this->register_prefilter("smarty_prefilter_i18n");

    $this->assign("web_root",$web_root);
  }

  function set_template($template)
  {
    $this->template_name = $template;
    if ( !empty($template)
      && file_exists(TEMPLATE_DIR.$template)) 
      {
	$this->template_dir = TEMPLATE_DIR.'/'.$template;
	$this->compile_dir = COMPILE_DIR.'/'.$template;
	$this->cache_dir = CACHE_DIR .'/' . $template;
	return 1;
      }
    return 0;
  }

  function fetch($_smarty_tpl_file, $_smarty_cache_id = null, $_smarty_compile_id = null, $_smarty_display = false) 
  {
    // We need to set the cache id and the compile id so a new script will be
    // compiled for each language. This makes things really fast ;-)
    $_smarty_compile_id = $this->language->getCurrentLanguage().'-'.$_smarty_compile_id;
    $_smarty_cache_id = $this->language->getCurrentLanguage().'-'.$_smarty_cache_id;

    // Now call parent method
    return parent::fetch( $_smarty_tpl_file, $_smarty_cache_id, $_smarty_compile_id, $_smarty_display );
  }

  function me()
  {
    if (!empty($_SERVER["PHP_SELF"])) {
      return $_SERVER["PHP_SELF"];
    } else {
      return $_SERVER["SCRIPT_NAME"];
    }
  }

  function display_page()
  {
    $this->assign("document_template" , "article.tpl");
    $article_medata = $this->read_file_metadata(WWW_DIR.$this->me());
    foreach ($article_medata  as $key => $value) {
      $this->assign($key,$value);
    }

    $this->__display_page($type);
  }

  function __display_page()
  {
    $cache_id = $this->me();
    $main_template = "main_template.tpl";

    if (!$this->is_cached($main_template,$cache_id)) 
    {
      # The page is not cached, so build tree, fetch data, ...

      $nav = new NavigationTree();
      $tree = $nav->build_navigation_tree($this->me());
      $this->assign('navigation_tree', $tree);
      $bar = $nav->build_navigation_bar($this->me());
      $this->assign('portal_navbar', $bar);
      $this->assign('current_page_level', $nav->current_page_level);

      $portal_tabs = new PortalTabs();
      $tabs = $portal_tabs->build_portal_tabs($this->me());
      $this->assign('portal_tabs', $tabs);
    }
    $this->display($main_template,$cache_id);
  }

  function read_file_metadata($file)
  {
    $file_info = array ();

    if (!is_file($file))
      return $file_info;

    $file_info['document_nav_title'] = basename($file);
    $file_info['document_filename'] = basename($file);
    $file_info['document_date'] = filemtime($file);
    $file_info['document_icon'] = "file_icon.gif";
    $file_info['document_size'] = filesize($file);

    $file = "$file.metadata";

    # Is a .metadata file exist, yes load it
    if (!file_exists($file))
      return $file_info;

    $fdmeta = fopen($file,"r");
    if (!$fdmeta)
      return $file_info;

    unset($file_info['document_nav_title']);
    $parse_headers = 1;
    while (!feof($fdmeta)) {
      if ($parse_headers) {
	$buffer = fgets($fdmeta,4096);
	if (ereg("^([a-zA-Z0-9_-]+): (.*)$", $buffer, $regs)) {
	  $key = $regs[1];
	  $file_info[$key] = $this->language->getTranslation(trim($regs[2]));
	} else {
	  $parse_headers = 0;
	}
      } else {
	$buffer .= fgets($fdmeta,4096);
      }
    }
    fclose($fdmeta);

    $file_info['document_content'] = $buffer;
    if (filemtime($file) > $file_info['date']) {
      $file_info['document_date'] = filemtime($file);
    }
    if (!isset($metadatas['document_nav_title'])) {
      $metadatas['document_nav_title'] = $metadatas['document_title'];
    }
    return $file_info;
  }

  function cmp_title($a, $b) 
  {
    return strcasecmp($a['document_nav_title'],$b['document_nav_title']);
  }



} /* class Page */

?>
