<?php

/**
* PageFolder.php
* @author Luc Saillard <luc@saillard.org>
* @version 0.1
*
* PageFolder to display a content of a folder when no index.php or index.html
* is present
*/


// Create a wrapper class extended from Smarty
class PageFolder extends Page
{
  function PageFolder()
  {
    $this->Page();
  }

  function display_page()
  {
    $this->assign("document_template" , "folder.tpl");
    $folder = dirname(WWW_DIR.$this->me());
    $this->fill_folder_template($folder);

    $this->__display_page();
  }

  /*
   * This function scan a directory, and put all variables
   * in a array to be use by the folder.tpl template
   */
  function fill_folder_template($folder=".")
  {
    $folder_list = array ();

    if (!is_dir($folder))
      return;

    $current_folder_metadata = $this->read_folder_metadata($folder);
    if (!isset($current_folder_metadata['document_title'])) {
      $current_folder_metadata['document_title'] = dirname($this->me());
    }
    foreach ($current_folder_metadata  as $key => $value) {
      $this->assign($key,$value);
    }

    $fd = opendir($folder);
    if (!$fd)
      return;

    $folders = array();
    $files   = array();

    # Scan the directory and sort folders and files using two lists
    while (false !== ($entry = readdir($fd))) {
      if( $entry[0] == '.'
       || ereg("\.(php|metadata)$",$entry)) {
	# Skip, file or directory that begins with a .
	# Skip link, directory, ..., and metadata file
	continue;
      }
      if (is_dir($entry)) {
	$folders[] = $this->read_folder_metadata("$folder/$entry");
      } else if (is_file($entry)) {
	$files[] = $this->read_file_metadata("$folder/$entry");
      }
    }
    closedir($fd);

    usort($folders,array($this,"cmp_title"));
    usort($files,array($this,"cmp_title"));

    $this->assign("folders_list", $folders);
    $this->assign("files_list", $files);
  }

  /*
   * The format us by the file is similar to http protocol.
   * 2 sections: headers and text
   * text section is separated from the header by an empty line.
   * header is separated by newline and contains "key: value" format
   * key will be the variable name use in the template
   * value can be any value
   */
  function read_folder_metadata($folder)
  {
    $metadatas = array();

    if (!is_dir($folder))
      return $metadatas;

    $file = "$folder/index.php.metadata";
    $metadatas['document_nav_title'] = end(explode('/',dirname($file)));
    $metadatas["document_date"] = filemtime(dirname($file));
    $metadatas['document_filename'] = end(explode('/',$folder)).'/';

    if (!file_exists($file))
      return $metadatas;

    $fd = fopen($file,"r");
    if (!$fd)
      return $metadatas;

    unset($metadatas['document_nav_title']);
    while (!feof($fd)) {
      $buffer = fgets($fd,4096);
      if (ereg("^([a-zA-Z0-9_-]+): (.*)$", $buffer, $regs)) {
	$key = $regs[1];
	$metadatas[$key] = $this->language->getTranslation(trim($regs[2]));
      } else {
	break;
      }
    }
    fclose($fd);
    $metadatas["document_date"] = filemtime($file);
    if (!isset($metadatas['document_nav_title'])) {
      $metadatas['document_nav_title'] = $metadatas['document_title'];
    }
    return $metadatas;
  }


  function read_file_metadata($file)
  {
    $file_info = array ();

    if (!is_file($file))
      return $file_info;

    $file_info['document_nav_title'] = basename($file);
    $file_info['document_filename'] = basename($file);
    $file_info['document_date'] = filemtime($file);
    $file_info['document_icon'] = "file_icon.gif";
    $file_info['document_size'] = filesize($file);
    if ((time() - $file_info['document_date']) < (3600*48)) {
      $file_info['highlight'] = 1;
    }

    $file = "$file.metadata";

    # Is a .metadata file exist, yes load it
    if (!file_exists($file))
      return $file_info;

    $fdmeta = fopen($file,"r");
    if (!$fdmeta)
      return $file_info;

    unset($file_info['document_nav_title']);
    $parse_headers = 1;
    while (!feof($fdmeta)) {
      if ($parse_headers) {
	$buffer = fgets($fdmeta,4096);
	if (ereg("^([a-zA-Z0-9_-]+): (.*)$", $buffer, $regs)) {
	  $key = $regs[1];
	  $file_info[$key] = $this->language->getTranslation(trim($regs[2]));
	} else {
	  $parse_headers = 0;
	}
      } else {
	$buffer .= fgets($fdmeta,4096);
      }
    }
    fclose($fdmeta);

    $file_info['document_content'] = $buffer;
    if (filemtime($metadata) > $file_info['date']) {
      $file_info['document_date'] = filemtime($metadata);
    }
    if (!isset($metadatas['document_nav_title'])) {
      $file_info['document_nav_title'] = $file_info['document_title'];
    }
    if ((time() - $file_info['document_date']) < (3600*48)) {
      $file_info['highlight'] = 1;
    }
    return $file_info;
  }

  function cmp_title($a, $b) 
  {
    return strcasecmp($a['document_nav_title'],$b['document_nav_title']);
  }


} /* class PageFolder */

?>
